"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthorizedGenericClient = exports.GenericClient = exports.classJSONRPCServerException = exports.JSONRPCException = void 0;
const axios_1 = __importDefault(require("axios"));
const DEFAULT_TIMEOUT = 10000;
class JSONRPCException extends Error {
    constructor({ name, code, message, error }) {
        super(message);
        this.name = name;
        this.code = code;
        this.error = error;
    }
}
exports.JSONRPCException = JSONRPCException;
class classJSONRPCServerException extends Error {
}
exports.classJSONRPCServerException = classJSONRPCServerException;
class GenericClient {
    constructor({ url, token, module, timeout }) {
        this.url = url;
        this.token = token || null;
        this.module = module;
        this.timeout = timeout || DEFAULT_TIMEOUT;
    }
    makePayload(method, param) {
        return {
            version: '1.1',
            method: this.module + '.' + method,
            id: String(Math.random()).slice(2),
            params: param
        };
    }
    callFunc(func, param) {
        return __awaiter(this, void 0, void 0, function* () {
            const headers = {
                'Content-Type': 'application/json',
                'Accept': 'application/json'
            };
            if (this.token) {
                headers['Authorization'] = this.token;
            }
            const params = this.makePayload(func, param);
            const response = yield axios_1.default.post(this.url, params, {
                headers,
                timeout: this.timeout
            });
            return response.data.result;
        });
    }
}
exports.GenericClient = GenericClient;
class AuthorizedGenericClient extends GenericClient {
    constructor(params) {
        super(params);
        if (!params.token) {
            throw new Error('Authorized client requires token');
        }
        this.token = params.token;
    }
    callFunc(func, param) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = this.makePayload(func, param);
            const headers = {
                'Content-Type': 'application/json',
                'Accept': 'application/json',
                'Authorization': this.token,
            };
            const response = yield axios_1.default.post(this.url, params, {
                headers,
                timeout: this.timeout
            });
            return response.data.result;
        });
    }
}
exports.AuthorizedGenericClient = AuthorizedGenericClient;
//# sourceMappingURL=GenericClient.js.map