"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicServiceClient = void 0;
const ServiceWizard_1 = require("../coreServices/ServiceWizard");
const ServiceClient_1 = require("./ServiceClient");
const Cache_1 = __importDefault(require("../Cache"));
const ITEM_LIFETIME = 1800000;
const MONITORING_FREQUENCY = 60000;
const WAITER_TIMEOUT = 30000;
const WAITER_FREQUENCY = 100;
var moduleCache = new Cache_1.default({
    itemLifetime: ITEM_LIFETIME,
    monitoringFrequency: MONITORING_FREQUENCY,
    waiterTimeout: WAITER_TIMEOUT,
    waiterFrequency: WAITER_FREQUENCY
});
class DynamicServiceClient extends ServiceClient_1.ServiceClient {
    constructor(params) {
        super(params);
        this.serviceDiscoveryModule = 'ServiceWizard';
        const { version } = params;
        this.version = version || null;
        if (this.version === 'auto') {
            this.version = null;
        }
        this.serviceDiscoveryURL = params.url;
    }
    moduleId() {
        let moduleId;
        if (!this.version) {
            moduleId = this.module + ':auto';
        }
        else {
            moduleId = this.module + ':' + this.version;
        }
        return moduleId;
    }
    getCached(fetcher) {
        return moduleCache.getItemWithWait({
            id: this.moduleId(),
            fetcher: fetcher
        });
    }
    lookupModule() {
        return __awaiter(this, void 0, void 0, function* () {
            const moduleInfo = yield this.getCached(() => __awaiter(this, void 0, void 0, function* () {
                const client = new ServiceWizard_1.ServiceWizardClient({
                    url: this.serviceDiscoveryURL,
                    token: this.token,
                    timeout: this.timeout
                });
                const status = yield client.get_service_status({
                    module_name: this.module,
                    version: this.version
                });
                return status;
            }));
            this.module = moduleInfo.module_name;
            this.url = moduleInfo.url;
            return moduleInfo;
        });
    }
    callFunc(funcName, params) {
        const _super = Object.create(null, {
            callFunc: { get: () => super.callFunc }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield this.lookupModule();
            return _super.callFunc.call(this, funcName, params);
        });
    }
    callFuncEmptyResult(funcName, params) {
        const _super = Object.create(null, {
            callFuncEmptyResult: { get: () => super.callFuncEmptyResult }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield this.lookupModule();
            return _super.callFuncEmptyResult.call(this, funcName, params);
        });
    }
}
exports.DynamicServiceClient = DynamicServiceClient;
//# sourceMappingURL=DynamicServiceClient.js.map